package com.logicbig.example;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.SqlParameterValue;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSourceUtils;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.jdbc.object.GenericSqlQuery;
import org.springframework.jdbc.object.SqlQuery;
import org.springframework.jdbc.object.SqlUpdate;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Arrays;
import java.util.List;

public class SqlUpdateExample {

    @Autowired
    private DataSource dataSource;
    private SqlUpdate updateSql;
    private SqlUpdate insertSql;
    private GenericSqlQuery<Person> loadSqlQuery;

    @PostConstruct
    private void postConstruct() {
        initLoadSql();
        initInsertSql();
        initUpdateSql();
    }

    private void initInsertSql() {
        insertSql = new SqlUpdate();
        insertSql.setDataSource(dataSource);
        insertSql.setSql("insert into PERSON (first_Name, last_Name, address) values (?, ?, ?)");
        insertSql.declareParameter(new SqlParameterValue(Types.VARCHAR, "first_Name"));
        insertSql.declareParameter(new SqlParameterValue(Types.VARCHAR, "last_Name"));
        insertSql.declareParameter(new SqlParameterValue(Types.VARCHAR, "address"));
    }

    private void initUpdateSql() {
        updateSql = new SqlUpdate();
        updateSql.setDataSource(dataSource);
        updateSql.setSql("update PERSON set address = ? where id = ?");
        updateSql.declareParameter(new SqlParameterValue(Types.VARCHAR, "address"));
        updateSql.declareParameter(new SqlParameterValue(Types.BIGINT, "id"));
    }

    private void initLoadSql() {
        loadSqlQuery = new GenericSqlQuery<>();
        loadSqlQuery.setDataSource(dataSource);
        loadSqlQuery.setSql("select * from Person");
        loadSqlQuery.setRowMapper(createPersonRowMapper());
    }

    private RowMapper<Person> createPersonRowMapper() {
        return (rs, rowNum) -> {
            return toPerson(rs);
        };
    }

    private Person toPerson(ResultSet rs) throws SQLException {
        Person person = new Person();
        person.setId(rs.getLong("ID"));
        person.setFirstName(rs.getString("FIRST_NAME"));
        person.setLastName(rs.getString("LAST_NAME"));
        person.setAddress(rs.getString("ADDRESS"));
        return person;
    }

    public void runExmaple() {
        //inserting records
        insertSql.update("Dana", "Whitley", "464 Gorsuch Drive");
        insertSql.update("Robin", "Cash", "64 Zella Park");
        System.out.println("Records inserted");

        //loading all person
        List<Person> list = loadSqlQuery.execute();
        System.out.printf("loaded: %s%n", list);

        //updating
        updateSql.update("XYZ", 1);
        System.out.println("Record updated");

        //loading again
        System.out.println("Loading again after update");
        List<Person> list2 = loadSqlQuery.execute();
        System.out.printf("loaded: %s%n", list2);
    }
}